unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, SQLDB, DB, SQLite3Conn, Forms, Controls, Graphics, Dialogs,
  DBCtrls, DBGrids, ExtCtrls;

type

  { TForm1 }

  TForm1 = class(TForm)
    DataSource: TDataSource;
    DBGrid: TDBGrid;
    DBMemo: TDBMemo;
    DBNavigator: TDBNavigator;
    Splitter: TSplitter;
    Connection: TSQLite3Connection;
    Query: TSQLQuery;
    Transaction: TSQLTransaction;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FAbort: Boolean;
    FDummyBar: String;
    FLastRec: Integer;
    procedure ClearAbort(Data: PtrInt);
    procedure QueryAfterScroll(DataSet: TDataSet);
    procedure QueryBeforeScroll(DataSet: TDataSet);
    procedure ScrollBack(Data: PtrInt);
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}

const
  IndexId  = 0;
  IndexFoo = 1;
  IndexBar = 2;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  FAbort := False;
  Connection.DatabaseName := GetCurrentDir + PathDelim + 'test.sql';
  Connection.HostName := 'localhost';
  Connection.Connected := True;

  Query.SQL.Text:=
    'CREATE TABLE IF NOT EXISTS test (' +
    'id INTEGER PRIMARY KEY AUTOINCREMENT, ' +
    'foo VARCHAR, ' +
    'bar VARCHAR);';
  Query.ExecSQL;
  Transaction.Commit;

  Query.AfterScroll  := @QueryAfterScroll;
  Query.BeforeScroll := @QueryBeforeScroll;

  Query.SQL.Text := 'SELECT id, foo, bar FROM test;';
  Query.PacketRecords := 100;
  Query.Open;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Query.ApplyUpdates;
  Transaction.Commit;
end;

procedure TForm1.QueryAfterScroll(DataSet: TDataSet);
begin
  if FAbort then
  begin
    Application.QueueAsyncCall(@ClearAbort, 0);
    Exit;
  end;
  FDummyBar := Query.Fields[IndexBar].AsString;
  FLastRec := Query.RecNo;
end;

procedure TForm1.ClearAbort(Data: PtrInt);
begin
  FAbort := False;
end;

procedure TForm1.QueryBeforeScroll(DataSet: TDataSet);
var
  LBar: String;
begin
  if FAbort then Exit;
  LBar := Query.Fields[IndexBar].AsString;
  if LBar <> FDummyBar then
    case QuestionDlg('Scroll Dataset', 'Daten von "bar" wurde verändert. Änderung übernehmen?',
                     mtConfirmation, [mrYes, 'Ja', mrNo, 'Nein', mrAbort, 'Abbruch'], 0)
    of
      mrYes:;
      mrNo:
        begin
          Query.Edit;
          Query.Fields[IndexBar].AsString := FDummyBar;
          Query.Post;
        end;
      else
        begin
          FAbort := True;
          Application.QueueAsyncCall(@ScrollBack, 0);
        end;
    end;
end;

procedure TForm1.ScrollBack(Data: PtrInt);
begin
  Query.RecNo := FLastRec;
  FAbort := False;
end;

end.

